#include <avr/io.h>
#include <util/delay.h>
#include <avr/interrupt.h>
#include <stdlib.h> 
#include <stdio.h> 
#include <stdint.h>
#include "Nokia5110/image.h"

#include "Nokia5110/Nokia5110.h"
#include "dht.h"
#include "temt6000.h"

static int cursor = 0;
static int old_cursor = 0;
int e; //ENTER

struct flower {
	char name[8];
	int min_temp;
	int max_temp;
	int light;
	int hummidity;
};

static struct flower flowers[4];


// Defines 
#define LCD_WIDTH  84
#define LCD_HEIGHT 48

#define BLOCK_SIZE 4

#define LIGHT_SENSOR 0
#define HUMIDITY_SOIL_SENSOR 1

uint8_t channelZero = LIGHT_SENSOR;

/* Valoare citita canal 0*/
volatile uint16_t ADC_value;


ISR(PCINT1_vect)
{
    if ((PINB & (1 << PB3)) == 0)
    {
        cursor--;	
 		if(cursor == -1)
 			cursor = 3;

 		LCD_write_string(0, cursor, "->");
 		_delay_ms(50);
 		if(old_cursor != cursor)
 			LCD_write_string(0, old_cursor, "  ");
	

 		old_cursor = cursor;

    }

     if ((PINB & (1 << PB4)) == 0)
    {
        cursor++;	
 		if(cursor == 4)
 			cursor = 0;

 		LCD_write_string(0, cursor, "->");
 		_delay_ms(50);

 		if(old_cursor != cursor)
 			LCD_write_string(0, old_cursor, "  ");
	

 		old_cursor = cursor;

    }
    _delay_ms(100);
}

void ADC_init(void)
{
    ADMUX = 0;
    ADMUX |= (1 << REFS0);
    ADCSRA = (1 << ADEN) | (7 << ADPS0);

}

/*
    Returns the result of an ADC conversion. 
    Starts a conversion and waits for the result when not using interrupts.
*/
int ADC_get(uint8_t channel)
{
    ADMUX = (ADMUX & ~(0x1f << MUX0)) | channel;

    ADCSRA |= (1 << ADSC);
    while(ADCSRA & (1 << ADSC));

    return ADC;
}

/*
    Converts a value given by the ADC to volts.
*/
double ADC_voltage(int raw)
{
    (void)raw;

    return raw * 5 / 1024.;
}

long map( long x,  long in_min,  long in_max,  long out_min,  long out_max)
{
  return (x - in_min) * (out_max - out_min) / (in_max - in_min) + out_min;
}

void buttons_init()
{
    DDRB &= ~(1 << PB2);
    PORTB |= (1 << PB2);

    DDRB &= ~(1 << PB3);
    PORTB |= (1 << PB3);

    DDRB &= ~(1 << PB4);
    PORTB |= (1 << PB4);
    // Activez PIN CHANGE INTERRUPT ENABLE pentru PCINT15:8 (portul B)
    // Orice PIN setat pt intrerupere va semnala programul
    PCICR |= (1 << PCIE1);

    //Activez intreruperea pe PB3 si PB4
    PCMSK1 |= (1 << PCINT11);
    PCMSK1 |= (1 << PCINT12);
   
}

int enter()
{
	if ((PINB & (1 << PB2)) == 0) 
        return 1;

    return 0;
}


int main(void) 
{
	int8_t temperature = 0;
	int8_t hummidity = 0;
	char str[8];
	uint16_t adc = 0;
	char printbuff[100];
	char value_voltage[32];
	char value_lux[32];
	char value_hsoil[32];
	long h_soil_percent;
	int ef = 0;

	
	LCD_init();    
	LCD_clear();

	sei();
	buttons_init();

	

	ADC_init();

	LCD_draw_bmp_pixel(0,0,image1,LCD_WIDTH,LCD_HEIGHT);
	LCD_write_string(0,2,"E");
	LCD_write_string(0,3,"V");
	LCD_write_string(0,4,"E");
	LCD_write_string(0,5,"R");

	LCD_write_string(76,1,"B");
	LCD_write_string(76,2,"L");
	LCD_write_string(76,3,"O");
	LCD_write_string(76,4,"O");
	LCD_write_string(76,5,"M");

	while(1) {
		e = enter();
		if(e == 1){
			break;
		}
	}
	e = 0;
	LCD_clear();
	LCD_write_string(13, 0, "Camellia");
	LCD_write_string(13, 1, "Freesia");
	LCD_write_string(13, 2, "Petunia");
	LCD_write_string(13, 3, "Primula");


	while(1) {
		e = enter();
		if(e == 1){
			break;
		}
	}
	e = 0;

	_delay_ms(5000);
	LCD_clear();

	while(1) {

		double voltage_lum = ADC_voltage(ADC_get(LIGHT_SENSOR));
		int h_soil = ADC_get(HUMIDITY_SOIL_SENSOR);
		//h_soil = 1023 - h_soil;
		h_soil_percent = map(h_soil, 110, 750, 0, 100);

		double lux = temt6000_getlux(voltage_lum);


		dtostrf(voltage_lum,3,5,value_voltage);
		dtostrf(lux,3,2,value_lux);
		itoa((int)h_soil_percent,value_hsoil,10);

		_delay_ms(50);

		dht_gettemperaturehumidity(&temperature,&hummidity);
		_delay_ms(50);

		itoa((int8_t)temperature,str,10);
		//dtostrf(temperature,3,2,str);


		LCD_write_string(5, 5, str);
		LCD_write_string(40, 2,value_lux);
		LCD_write_string(5, 2,value_hsoil);




    }
	
	LCD_clear();

	return 0;
}



